#!/usr/bin/env python

from __future__ import print_function
import sys

# The current version of the system.  Format is #.#.#[-DEV].
version = '1.2.5'

# Require Python 2.6 or higher or Python 3.1 or higher
if (sys.version_info[:2] < (2, 6)) or ((sys.version_info[0] == 3) and sys.version_info[:2] < (3, 1)):
    raise ValueError('''PyXB requires:
  Python2 version 2.6 or later; or
  Python3 version 3.1 or later
(You have %s.)''' % (sys.version,))

import os
import stat
import re
import datetime
import logging

from distutils.core import setup, Command

# Stupid little command to automatically update the version number
# where it needs to be updated.
class update_version (Command):
    # Brief (40-50 characters) description of the command
    description = "Substitute @VERSION@ in relevant files"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [ ]
    boolean_options = [ ]

    # Files in the distribution that need to be rewritten when the
    # version number changes
    files = ( 'README.txt', 'pyxb/__init__.py', 'doc/conf.py' )

    # The substitutions (key braced by @ signs)
    substitutions = { 'VERSION' : version,
                      'THIS_YEAR' : datetime.date.today().strftime('%Y'),
                      'SHORTVERSION' : '.'.join(version.split('.')[:2]) }

    def initialize_options (self):
        pass

    def finalize_options (self):
        pass

    def run (self):
        for f in self.files:
            text = open('%s.in' % (f,)).read()
            for (k, v) in self.substitutions.items():
                text = text.replace('@%s@' % (k,), v)
            os.chmod(f, os.stat(f)[stat.ST_MODE] | stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH)
            open(f,'w').write(text)
            os.chmod(f, os.stat(f)[stat.ST_MODE] & ~(stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH))

class test (Command):

    # Brief (40-50 characters) description of the command
    description = "Run all unit tests found in testdirs"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [ ( 'testdirs=', None, 'colon separated list of directories to search for tests' ),
                     ( 'trace-tests', 'v', 'trace search for tests' ),
                     ( 'inhibit-output', 'q', 'inhibit test output' ),
                     ]
    boolean_options = [ 'trace-tests', 'inhibit-output' ]

    def initialize_options (self):
        self.trace_tests = None
        self.inhibit_output = None
        self.testdirs = 'tests'

    def finalize_options (self):
        pass

    # Regular expression that matches unittest sources
    __TestFile_re = re.compile('^test.*\.py$')

    def run (self):
        # Make sure log messages are supported
        logging.basicConfig()

        # Walk the tests hierarchy looking for tests
        dirs = self.testdirs.split(':')
        tests = [ ]
        while dirs:
            dir = dirs.pop(0)
            if self.trace_tests:
                print('Searching for tests in %s' % (dir,))
            for f in os.listdir(dir):
                fn = os.path.join(dir, f)
                statb = os.stat(fn)
                if stat.S_ISDIR(statb[0]):
                    dirs.append(fn)
                elif self.__TestFile_re.match(f):
                    tests.append(fn)

        number = 0
        import sys
        import traceback
        import unittest
        import types

        # Import each test into its own module, then add the test
        # cases in it to a complete suite.
        loader = unittest.defaultTestLoader
        suite = unittest.TestSuite()
        used_names = set()
        for fn in tests:
            stage = 'compile'
            try:
                # Assign a unique name for this test
                test_name = os.path.basename(fn).split('.')[0]
                test_name = test_name.replace('-', '_')
                number = 2
                base_name = test_name
                while test_name in used_names:
                    test_name = '%s%d' % (base_name, number)
                    number += 1

                # Read the test source in and compile it
                rv = compile(open(fn).read(), test_name, 'exec')
                state = 'evaluate'

                # Make a copy of the globals array so we don't
                # contaminate this environment.
                g = globals().copy()

                # The test cases use __file__ to determine the path to
                # the schemas
                g['__file__'] = fn

                # Create a module into which the test will be evaluated.
                module = types.ModuleType(test_name)

                # The generated code uses __name__ to look up the
                # containing module in sys.modules.
                g['__name__'] = test_name
                sys.modules[test_name] = module

                # Import the test into the module, making sure the created globals look like they're in the module.
                eval(rv, g)
                module.__dict__.update(g)

                # Find all subclasses of unittest.TestCase that were
                # in the test source and add them to the suite.
                for (nm, obj) in g.items():
                    if (type == type(obj)) and issubclass(obj, unittest.TestCase):
                        suite.addTest(loader.loadTestsFromTestCase(obj))
                if self.trace_tests:
                    print('%s imported' % (fn,))
            except Exception as e:
                print('%s failed in %s: %s' % (fn, stage, e))
                raise

        # Run everything
        verbosity = 1
        if self.trace_tests:
            verbosity = 2
        elif self.inhibit_output:
            # Don't know how to do this for real
            verbosity = 0
        runner = unittest.TextTestRunner(verbosity=verbosity)
        runner.run(suite)

import glob
import sys
import pyxb.utils.utility

packages = [
        'pyxb', 'pyxb.namespace', 'pyxb.binding', 'pyxb.utils', 'pyxb.xmlschema',
        "pyxb.bundles"
        ]
package_data = {}

init_re = re.compile('^__init__\.py$')
wxs_re = re.compile('^.*\.wxs$')

setup_path = os.path.dirname(__file__)
bundle_base = os.path.join(setup_path, 'pyxb', 'bundles')
possible_bundles = []
try:
    possible_bundles.extend(os.listdir(bundle_base))
except OSError as e:
    print("Directory %s bundle search failed: %s" % (bundle_base, e))
for possible_bundle in possible_bundles:
    bundle_root = os.path.join(bundle_base, possible_bundle)
    if not os.path.isdir(bundle_root):
        continue
    b_packages = []
    b_data = { }
    for fp in pyxb.utils.utility.GetMatchingFiles('%s//' % (bundle_root,), init_re):
        bundle_path = os.path.dirname(os.path.normpath(fp))
        try:
            package_relpath = os.path.relpath(bundle_path, setup_path)
        except AttributeError as e:
            package_relpath = bundle_path
            if setup_path and '.' != setup_path:
                prefix_path = setup_path + os.path.sep
                if not package_relpath.startswith(prefix_path):
                    print("Unable to determine relative path from %s to %s installation" % (setup_path, bundle_path))
                    sys.exit(1)
                package_relpath = package_relpath[len(prefix_path):]
        package = package_relpath.replace(os.path.sep, '.')
        b_packages.append(package)
        wxs_files = [os.path.basename(_f) for _f in pyxb.utils.utility.GetMatchingFiles(bundle_path, wxs_re) ]
        if wxs_files:
            b_data[package] = wxs_files
    if 0 < len(b_data):
        print('Found bundle in %s' % (bundle_root,))
        packages.extend(b_packages)
        package_data.update(b_data)

setup(name='PyXB',
      description = 'PyXB ("pixbee") is a pure Python package that generates Python source code for classes that correspond to data structures defined by XMLSchema.',
      author='Peter A. Bigot',
      author_email='pabigot@users.sourceforge.net',
      url='http://pyxb.sourceforge.net',
      # Also change in README.TXT, pyxb/__init__.py, and doc/conf.py
      version=version,
      license='Apache License 2.0',
      long_description='''PyXB is a pure `Python <http://www.python.org>`_ package that generates
Python code for classes that correspond to data structures defined by
`XMLSchema <http://www.w3.org/XML/Schema>`_.  In concept it is similar to
`JAXB <http://en.wikipedia.org/wiki/JAXB>`_ for Java and `CodeSynthesis XSD
<http://www.codesynthesis.com/products/xsd/>`_ for C++.

The major goals of PyXB are:

* Provide a generated Python interface that is "Pythonic", meaning similar
  to one that would have been hand-written:

  + Attributes and elements are Python properties, with name conflicts
    resolved in favor of elements
  + Elements with maxOccurs larger than 1 are stored as Python lists
  + Bindings for type extensions inherit from the binding for the base type
  + Enumeration constraints are exposed as class (constant) variables

* Support bi-directional conversion (document to Python and back)

* Allow easy customization of the generated bindings to provide
  functionality along with content

* Support all XMLSchema features that are in common use, including:

  + complex content models (nested all/choice/sequence)
  + cross-namespace dependencies
  + include and import directives
  + constraints on simple types
''',
      provides=[ 'PyXB' ],
      packages=packages,
      package_data=package_data,
      # I normally keep these in $purelib, but distutils won't tell me where that is.
      # We don't need them in the installation anyway.
      #data_files= [ ('pyxb/standard/schemas', glob.glob(os.path.join(*'pyxb/standard/schemas/*.xsd'.split('/'))) ) ],
      scripts=[ 'scripts/pyxbgen', 'scripts/pyxbwsdl', 'scripts/pyxbdump' ],
      cmdclass = { 'test' : test,
                   'update_version' : update_version },
      classifiers = [ 'Development Status :: 5 - Production/Stable'
                      , 'Intended Audience :: Developers'
                      , 'License :: OSI Approved :: Apache Software License'
                      , 'Topic :: Software Development :: Code Generators'
                      , 'Topic :: Text Processing :: Markup :: XML'
                      , 'Programming Language :: Python :: 2'
                      , 'Programming Language :: Python :: 2.6'
                      , 'Programming Language :: Python :: 2.7'
                      , 'Programming Language :: Python :: 3'
                      # Skip 3.0 because it doesn't know how to be built with hashlib support
                      , 'Programming Language :: Python :: 3.1'
                      , 'Programming Language :: Python :: 3.2'
                      , 'Programming Language :: Python :: 3.3'
                      , 'Programming Language :: Python :: 3.4'
                      ] )



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_boolean (unittest.TestCase):
    def testTrue (self):
        self.assertTrue(xsd.boolean(True))
        self.assertTrue(xsd.boolean("true"))
        self.assertTrue(xsd.boolean(1))
        self.assertTrue(xsd.boolean("1"))

    def testFalse (self):
        self.assertFalse(xsd.boolean(False))
        self.assertFalse(xsd.boolean("false"))
        self.assertFalse(xsd.boolean(0))
        self.assertFalse(xsd.boolean("0"))
        self.assertFalse(xsd.boolean())

    def testInvalid (self):
        self.assertRaises(SimpleTypeValueError, xsd.boolean, "True")
        self.assertRaises(SimpleTypeValueError, xsd.boolean, "FALSE")

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import xml.dom.minidom
import pyxb.utils.domutils
from cablelabs import core, offer, title, vod30
import custom

pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(core.Namespace, 'core')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(offer.Namespace, 'offer')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(title.Namespace, 'title')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(vod30.Namespace, 'vod30')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(custom.Namespace, 'custom')

adi3 = vod30.ADI3()

# Create an instance of ContentGroupType (which extends AssetType). It
# is not associated with an element.
cgt = offer.ContentGroupType()

# The thing we want to use as an extension
venue = custom.Venue('some multiplex')

# Give it an extension value. The Ext element in AssetType contains
# only wildcards; you can put whatever you want into it, but at the
# Ext Python attribute level it will have no structure.  You do need
# an instance of ExtType to hold the content.
ext = core.ExtType(venue)

# You can uncomment this to see what happens if you skip the
# core.ExtType wrapper
#ext = venue

# The assignment of a value to the Ext member is validated.
try:
    cgt.Ext = ext
except pyxb.ValidationError as e:
    print(e.details())
    raise

# Unfortunately cgt with just an Ext element does not validate.  Each
# of the following assignments comes in reaction to the nice error
# messages generated when a ValidationError is raised below.  Try
# commenting each out to see what happens.
cgt.uriId = 'urn:one'
cgt.TitleRef = core.AssetRefType(uriId='urn:aMovie')
cgt.PosterRef.append(core.AssetRefType(uriId='urn:aPoster'))

# Now you can add the instance to the description and convert it to an
# XML document.
adi3.ContentGroup.append(cgt)
try:
    xmls = adi3.toDOM().toprettyxml()
except pyxb.ValidationError as e:
    print(e.details())
    raise

# Show off the pretty document
print(xmls)

# Let's convert it back.  This showed a different misunderstanding of
# the schema too complicated to replicate here.
try:
    instance = core.CreateFromDocument(xmls)
except pyxb.ValidationError as e:
    print(e.details())
    raise

ivenue = instance.ContentGroup[0].Ext.wildcardElements()[0]
assert isinstance(ivenue, type(venue))
# NB: Python 2.6.8 and Python 2.7.3 have whitespace differences in
# text emitted by toprettyxml() which results in failure of
# non-stripped comparison in 2.6.8.
assert ivenue.strip() == venue.strip()

# Since the Ext element holds its content as an unbounded number of
# wildcards, you can just add stuff to it.
cgt.Ext.append(custom.Venue('another location'))
cgt.Ext.append(custom.Venue('and yet another'))

xmls = adi3.toDOM().toprettyxml()
print(xmls)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
           elementFormDefault="qualified">
    <xs:complexType mixed="true" name="tsAnyType">
        <xs:sequence>
            <xs:any maxOccurs="unbounded" minOccurs="0" namespace="##any" processContents="lax"/>
        </xs:sequence>
    </xs:complexType>

    <xs:element name="OpaqueData">
        <xs:complexType>
            <xs:sequence>
                <xs:element name="v" type="tsAnyType" minOccurs="1"/>
            </xs:sequence>
        </xs:complexType>
    </xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class Test_simple (unittest.TestCase):

    testxml = """<?xml version="1.0" encoding="UTF-8"?>
<OpaqueData
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    <v>test</v>
</OpaqueData>"""

    def test (self):
        txml = CreateFromDocument(self.testxml)
        xmld = '<?xml version="1.0" ?><OpaqueData><v>test</v></OpaqueData>'
        self.assertEqual(txml.toxml(), xmld)
        xmld = '<?xml version="1.0" encoding="utf-8"?><OpaqueData><v>test</v></OpaqueData>'.encode('utf-8')
        self.assertEqual(txml.toxml('utf-8'), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/po1.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

#open('code.py', 'w').write(code)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils

def ToDOM (instance, dom_support=None):
    return instance.toDOM(dom_support).documentElement

import unittest

class TestPO1 (unittest.TestCase):
    street_content = '''95 Main St.
Anytown, AS  12345-6789'''
    street_xmlt = six.u('<street>%s</street>') % (street_content,)
    street_xmld = street_xmlt.encode('utf-8')
    street_dom = pyxb.utils.domutils.StringToDOM(street_xmlt).documentElement

    address1_xmlt = six.u('<name>Customer</name><street>95 Main St</street>')
    address2_xmlt = six.u('<name>Sugar Mama</name><street>24 E. Dearling Ave.</street>')

    def tearDown (self):
        pyxb.RequireValidWhenGenerating(True)
        pyxb.RequireValidWhenParsing(True)

    def testPythonElementSimpleContent (self):
        elt = USAddress._ElementMap['street'].elementBinding()(self.street_content)
        self.assertEqual(self.street_content, elt)
        self.assertEqual(ToDOM(elt).toxml("utf-8"), self.street_xmld)

    def testDOMElementSimpleContent (self):
        elt = USAddress._ElementMap['street'].elementBinding().createFromDOM(self.street_dom)
        self.assertEqual(ToDOM(elt).toxml("utf-8"), self.street_xmld)

    def testPythonElementComplexContent_Element (self):
        addr = USAddress(name='Customer', street='95 Main St')
        self.assertEqual('95 Main St', addr.street)
        addr = USAddress('Customer', '95 Main St')
        self.assertEqual('95 Main St', addr.street)
        addr.street = '43 West Oak'
        self.assertEqual('43 West Oak', addr.street)

    def testDOM_CTD_element (self):
        # NB: USAddress is a CTD, not an element.
        xmlt = six.u('<shipTo>%s</shipTo>') % (self.address1_xmlt,)
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        addr2 = USAddress.Factory(_dom_node=dom.documentElement)

    def testPurchaseOrder (self):
        po = purchaseOrder(shipTo=USAddress(name='Customer', street='95 Main St'),
                           billTo=USAddress(name='Sugar Mama', street='24 E. Dearling Ave'),
                           comment='Thanks!')
        xmld = ToDOM(po).toxml("utf-8")
        xml1t = '<ns1:purchaseOrder xmlns:ns1="http://www.example.com/PO1"><shipTo><name>Customer</name><street>95 Main St</street></shipTo><billTo><name>Sugar Mama</name><street>24 E. Dearling Ave</street></billTo><ns1:comment>Thanks!</ns1:comment></ns1:purchaseOrder>'
        xml1d = xml1t.encode('utf-8')
        self.assertEqual(xmld, xml1d)

        dom = pyxb.utils.domutils.StringToDOM(xmld)
        po2 = purchaseOrder.createFromDOM(dom.documentElement)
        self.assertEqual(xml1d, ToDOM(po2).toxml("utf-8"))
        loc = po2.shipTo._location()
        self.assertTrue((not isinstance(loc, pyxb.utils.utility.Locatable_mixin)) or (58 == loc.columnNumber))
        loc = po2.billTo.name._location()
        self.assertTrue((not isinstance(loc, pyxb.utils.utility.Locatable_mixin)) or (131 == loc.columnNumber))

        po2 = CreateFromDocument(xmld)
        self.assertEqual(xml1d, ToDOM(po2).toxml("utf-8"))
        loc = po2.shipTo._location()
        self.assertTrue((not isinstance(loc, pyxb.utils.utility.Locatable_mixin)) or (58 == loc.columnNumber))
        loc = po2.billTo.name._location()
        self.assertTrue((not isinstance(loc, pyxb.utils.utility.Locatable_mixin)) or (131 == loc.columnNumber))

        xml2t = '<purchaseOrder xmlns="http://www.example.com/PO1"><shipTo><name>Customer</name><street>95 Main St</street></shipTo><billTo><name>Sugar Mama</name><street>24 E. Dearling Ave</street></billTo><comment>Thanks!</comment></purchaseOrder>'
        xml2d = xml2t.encode('utf-8')
        bds = pyxb.utils.domutils.BindingDOMSupport()
        bds.setDefaultNamespace(Namespace)
        self.assertEqual(xml2d, ToDOM(po2, dom_support=bds).toxml("utf-8"))

    def testGenerationValidation (self):
        ship_to = USAddress('Robert Smith', 'General Delivery')
        po = purchaseOrder(ship_to)
        self.assertEqual('General Delivery', po.shipTo.street)
        self.assertTrue(po.billTo is None)

        self.assertTrue(pyxb.RequireValidWhenGenerating())
        self.assertRaises(pyxb.IncompleteElementContentError, po.toxml)
        try:
            pyxb.RequireValidWhenGenerating(False)
            self.assertFalse(pyxb.RequireValidWhenGenerating())
            xmlt = six.u('<ns1:purchaseOrder xmlns:ns1="http://www.example.com/PO1"><shipTo><street>General Delivery</street><name>Robert Smith</name></shipTo></ns1:purchaseOrder>')
            xmlta = six.u('<ns1:purchaseOrder xmlns:ns1="http://www.example.com/PO1"><shipTo><name>Robert Smith</name><street>General Delivery</street></shipTo></ns1:purchaseOrder>')
            xmlds = [ _xmlt.encode('utf-8') for _xmlt in (xmlt, xmlta) ]
            self.assertTrue(po.toxml("utf-8", root_only=True) in xmlds)
        finally:
            pyxb.RequireValidWhenGenerating(True)
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDocument, xmlt)
        self.assertTrue(pyxb.RequireValidWhenParsing())
        try:
            pyxb.RequireValidWhenParsing(False)
            self.assertFalse(pyxb.RequireValidWhenParsing())
            po2 = CreateFromDocument(xmlt)
        finally:
            pyxb.RequireValidWhenParsing(True)
        self.assertEqual('General Delivery', po2.shipTo.street)
        self.assertTrue(po2.billTo is None)

if __name__ == '__main__':
    unittest.main()



# Demonstrate alternatives for bindings customization
# Traditional customization
from raw.custom import *
import raw.custom as raw_custom

class ta0 (raw_custom.ta0):
    def xa0 (self):
        return 'extend ta0'
raw_custom.ta0._SetSupersedingClass(ta0)

class tc01 (raw_custom.tc01):
    def xc01 (self):
        return 'extend tc01'
raw_custom.tc01._SetSupersedingClass(tc01)

class tc02 (raw_custom.tc02, ta0):
    def xc02 (self):
        return 'extend tc02'
raw_custom.tc02._SetSupersedingClass(tc02)

# class tc03 left as original



# Demonstrate alternatives for bindings customization

# Introspection-based customization.
from raw.custom import *
import raw.custom as raw_custom

class ta04 (raw_custom.ta04):
    def xa04 (self):
        return 'extend ta04'
raw_custom.ta04._SetSupersedingClass(ta04)

import inspect

# Utility function to identify classes of interest
def _isSupersedable (cls):
    return inspect.isclass(cls) and issubclass(cls, pyxb.binding.basis._DynamicCreate_mixin)

def _injectClasses ():
    import sys
    import pyxb.binding.basis

    # All PyXB complex type definitions in the original module
    raw_classes = set([_o for (_, _o) in inspect.getmembers(raw_custom) if _isSupersedable(_o)])
    #print 'Original classes: %s' % (raw_classes,)

    # PyXB complex type definitions in this module that did not come
    # from the original import *.
    this_module = sys.modules[__name__]
    this_classes = set([_o for (_, _o) in inspect.getmembers(this_module) if _isSupersedable(_o) and _o not in raw_classes])
    this_classes_tuple = tuple(this_classes)
    #print 'This classes: %s' % (this_classes,)

    # Raw classes superseded by something in this module
    superseded_classes = set([ _o for _o in raw_classes if _o._SupersedingClass() in this_classes ])
    superseded_classes_tuple = tuple(superseded_classes)
    #print 'Superseded classes: %s' % (superseded_classes,)

    # Raw classes that are subclasses of something superseded by this
    # module, but that are not themselves superseded by this module
    need_supersedure_classes = set([_o for _o in raw_classes if issubclass(_o, superseded_classes_tuple) and _o not in superseded_classes])
    #print 'Need supersedure classes: %s' % (need_supersedure_classes,)

    # Add local definitions to supersede classes all of whose
    # ancestors have been superseded as necessary.
    while need_supersedure_classes:
        did_replacement = False
        new_need_supersedure_classes = set()
        for o in need_supersedure_classes:
            candidate = True
            # Build the new sequence of base classes while we check them.
            new_mro = []
            for super_o in o.__mro__:
                if super_o == o:
                    # Put the superseded class in its original position (probably first)
                    new_mro.append(o)
                    continue
                if super_o in need_supersedure_classes:
                    # Subclass of a class we haven't gotten to yet; put it off
                    candidate = False
                    break
                # Append the replacement or the original, as needed
                if super_o in superseded_classes:
                    new_mro.append(super_o._SupersedingClass())
                else:
                    new_mro.append(super_o)
            if not candidate:
                new_need_supersedure_classes.add(o)
                continue
            # Create a new class that subclasses the replacements
            name = o.__name__
            new_o = type(name, tuple(new_mro), o.__dict__.copy())
            # Install it in the module
            setattr(this_module, name, new_o)
            # Tell PyXB to use it as the superseding class
            o._SetSupersedingClass(new_o)
            # Record it so future passes will find it
            superseded_classes.add(o)
        assert need_supersedure_classes != new_need_supersedure_classes
        need_supersedure_classes = new_need_supersedure_classes

_injectClasses()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
# Undeclared XML namespace

import pyxb.binding.generate
import pyxb.utils.domutils
import pyxb.binding.datatypes as xs
from xml.dom import Node

import os.path
# <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xml="http://www.w3.org/XML/1998/namespace">

xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="tEnum">
    <xs:restriction base="xs:string">
      <xs:enumeration value="one"/>
      <xs:enumeration value="two"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="tUnion">
    <xs:union memberTypes="xs:int tEnum"/>
  </xs:simpleType>
  <xs:element name="union" type="tUnion"/>
  <xs:simpleType name="tListUnion">
    <xs:list itemType="tUnion"/>
  </xs:simpleType>
  <xs:element name="listUnion" type="tListUnion"/>
  <xs:complexType name="tStructure">
    <xs:sequence/>
    <xs:attribute name="attr" type="tUnion" use="required"/>
    <xs:attribute name="enum" type="tEnum" use="optional"/>
  </xs:complexType>
  <xs:element name="structure" type="tStructure"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

def setAttr (instance, value):
    instance.attr = value
    return instance

def setEnum (instance, value):
    instance.enum = value
    return instance

class TestTrac0037 (unittest.TestCase):
    def testElement (self):
        i = union(4)
        self.assertTrue(i.validateBinding())
        i = union('two')
        self.assertTrue(i.validateBinding())
        i = xs.int(53.42)
        i = union(52.43)
        i = listUnion([1, 3, 'one', 'two', 2])
        self.assertTrue(i.validateBinding())

    def testListMemberConversion (self):
        i = listUnion([])
        self.assertTrue(isinstance(i, list))
        self.assertEqual(0, len(i))
        i.extend([1, 3, 'one', 'two', 2])
        self.assertTrue(isinstance(i[0], xs.int))
        self.assertTrue(isinstance(i[2], tEnum))
        self.assertTrue(i.validateBinding())
        # setitem
        i[0] = 23
        self.assertTrue(isinstance(i[0], xs.int))
        self.assertEqual(23, i[0])
        self.assertTrue('one' in i)
        self.assertEqual(2, i.index('one'))
        self.assertEqual(5, len(i))
        i[2:2] = [ 37 ]
        self.assertEqual(6, len(i))
        self.assertEqual(37, i[2])
        self.assertTrue(isinstance(i[2], xs.int))
        self.assertEqual(3, i.index('one'))
        i[:2] = []
        self.assertEqual(4, len(i))
        self.assertEqual(37, i[0])
        i[1:] = [ 'two' ]
        self.assertEqual(2, len(i))
        self.assertEqual(1, i.index('two'))

    def testConstructor (self):
        self.assertRaises(pyxb.SimpleTypeValueError, structure, attr='three')
        i = structure(attr=4)
        self.assertTrue(i.validateBinding())
        i = structure(attr='two')
        self.assertTrue(i.validateBinding())

    def testSet (self):
        i = structure()
        setAttr(i, 4)
        self.assertEqual(i.attr, 4)
        self.assertTrue(isinstance(i.attr, int))
        setAttr(i, 'two')
        self.assertEqual(i.attr, 'two')
        self.assertTrue(isinstance(i.attr, tEnum))
        setEnum(i, 'one')
        self.assertEqual(i.attr, 'two')
        self.assertEqual(i.enum, 'one')
        self.assertTrue(isinstance(i.enum, tEnum))
        self.assertRaises(pyxb.SimpleTypeValueError, setAttr, i, 'three')
        self.assertRaises(pyxb.SimpleTypeValueError, setEnum, i, 4)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import gc

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xs:complexType name="color"/>
    <xs:complexType name="ecolor">
        <xs:complexContent>
            <xs:extension base="color">
               <xs:sequence>
                  <xs:element name="color" type="xs:string"/>
               </xs:sequence>
            </xs:extension>
        </xs:complexContent>
    </xs:complexType>
    <xs:complexType name="acolor">
        <xs:complexContent>
            <xs:extension base="color">
               <xs:attribute name="color" type="xs:string"/>
            </xs:extension>
        </xs:complexContent>
    </xs:complexType>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0194 (unittest.TestCase):
    def testAll (self):
        pass

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import xml.dom
import xml.dom.minidom
import pyxb.utils.domutils
from pyxb.utils import six
import pyxb.namespace

# Structure
#import DWML
#print 'Validating DWML'
#DWML.Namespace.validateSchema()
#print 'Validated DWML: types %s' % ("\n".join(six.iterkeys(DWML.Namespace.typeDefinitions()),)

dom = xml.dom.minidom.parse(open('NDFDgen.xml', 'rb'))
body_dom = dom.documentElement.firstChild.nextSibling.firstChild.nextSibling
print(body_dom)

# Service interface types
import ndfd

# WSDL
import pyxb.bundles.wssplat.wsdl11 as wsdl

uri_src = open('ndfdXML.wsdl')
doc = xml.dom.minidom.parse(uri_src)
spec = wsdl.definitions.createFromDOM(doc.documentElement, process_schema=True)

binding = spec.binding[0]
print(binding.name)
port_type = spec.portType[0]
print(port_type.name)
bop = binding.operationMap()[body_dom.localName]
print(bop.toxml("utf-8"))
pop = port_type.operationMap()[body_dom.localName]
print(pop.toxml("utf-8"))
input = pop.input
print(input.toxml("utf-8"))
print(type(input))
print(input.message)
im_en = input.message
print(im_en)
msg = im_en.message()
print(msg)
for p in msg.part:
    print(p.toxml("utf-8"))
msg_ns = pyxb.namespace.NamespaceForURI(body_dom.namespaceURI)
print('%s %s' % (body_dom.namespaceURI, msg_ns))

parts = msg.part
nodes = body_dom.childNodes

while parts and nodes:
    p = parts.pop(0)
    while nodes and (not (xml.dom.Node.ELEMENT_NODE == nodes[0].nodeType)):
        nodes.pop(0)
    assert nodes
    n = nodes.pop(0)
    if p.name != n.localName:
        print('Desynchronized: part %s expected node %s' % (p.name, n.localName))
        nodes.insert(0, n)
        continue
    print('%s %s' % (p.name, n.localName))

#print '%s yielded %s' msg_ns

#msg = spec.messageMap()
#print msg

#print req
#dom_support =  req.toDOM(pyxb.utils.domutils.BindingDOMSupport())
#dom_support.finalize()
#print dom_support.document().toxml("utf-8")



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import profile
import unittest

class TestIssue0048 (unittest.TestCase):
    def testProfile (self):
        amap = profile.AbstractFeatureBaseType._AttributeMap
        self.assertEqual(1, len(amap))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import unittest
import pyxb.binding.datatypes as xsd

class Test_gMonth (unittest.TestCase):

    def testBasic (self):
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.gMonth, 0)
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.gMonth, 13)
        v = xsd.gMonth('--10')
        self.assertEqual(v.month, 10)
        v = xsd.gMonth(10)
        self.assertEqual(v.month, 10)

    def testXSDLiteral (self):
        v = xsd.gMonth(10)
        self.assertEqual('--10', v.xsdLiteral())

    def testTimezoned (self):
        dt = xsd.gMonth('--08Z')
        self.assertEqual('1900-08-01T00:00:00+00:00', dt.isoformat())
        self.assertEqual('--08Z', dt.xsdLiteral())

        dt = xsd.gMonth('--08-14:00')
        self.assertEqual('1900-08-01T00:00:00-14:00', dt.isoformat())
        self.assertEqual('--08-14:00', dt.xsdLiteral())

        dt = xsd.gMonth('--08+14:00')
        self.assertEqual('1900-08-01T00:00:00+14:00', dt.isoformat())
        self.assertEqual('--08+14:00', dt.xsdLiteral())

    def XtestAccessor (self):
        v = xsd.gMonth(10)
        self.assertRaises((AttributeError, TypeError), getattr, v, 'year')
        #self.assertRaises((AttributeError, TypeError), getattr, v, 'month')
        self.assertRaises((AttributeError, TypeError), getattr, v, 'day')
        self.assertRaises((AttributeError, TypeError), setattr, v, 'year', 5)
        self.assertRaises((AttributeError, TypeError), setattr, v, 'month', 5)
        self.assertRaises((AttributeError, TypeError), setattr, v, 'day', 5)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import pyxb
import po1

xml = open('badcontent.xml').read()
try:
    order = po1.CreateFromDocument(xml, location_base='badcontent.xml')
except pyxb.ValidationError as e:
    print(e.details())



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
# Undeclared XML namespace
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.utils.domutils
from xml.dom import Node

import os.path

xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:attributeGroup name="required">
    <xs:attribute name="rattr" use="required" type="xs:int"/>
    <xs:attribute name="rattr_fixed" type="xs:int" fixed="30" use="required"/>
  </xs:attributeGroup>
  <xs:attributeGroup name="optional">
    <xs:attribute name="attr" type="xs:int"/>
    <xs:attribute name="attr_def" type="xs:int" default="10"/>
    <xs:attribute name="attr_fixed" type="xs:int" fixed="20"/>
  </xs:attributeGroup>
  <xs:complexType name="opt_struct">
    <xs:attributeGroup ref="optional"/>
  </xs:complexType>
  <xs:complexType name="req_struct">
    <xs:attributeGroup ref="required"/>
  </xs:complexType>
  <xs:element name="ireq_struct" type="req_struct"/>
  <xs:element name="iopt_struct" type="opt_struct"/>
  <xs:complexType name="opt_def">
    <!-- This does have three attributes; it just changes one of the ones it inherits -->
    <xs:complexContent>
      <xs:restriction base="opt_struct">
        <xs:attribute name="attr" type="xs:int" default="5"/>
      </xs:restriction>
    </xs:complexContent>
  </xs:complexType>
  <xs:element name="iopt_def" type="opt_def"/>
  <xs:complexType name="opt_pro">
    <xs:complexContent>
      <xs:restriction base="opt_struct">
        <xs:attribute name="attr" use="prohibited"/>
        <xs:attribute name="attr_def" use="prohibited"/>
      </xs:restriction>
    </xs:complexContent>
  </xs:complexType>
  <xs:element name="iopt_pro" type="opt_pro"/>
  <xs:complexType name="opt_rest">
    <xs:complexContent>
      <xs:restriction base="opt_struct">
        <xs:attribute name="attr" type="xs:byte"/>
      </xs:restriction>
    </xs:complexContent>
  </xs:complexType>
  <xs:element name="iopt_rest" type="opt_rest"/>

<!-- TEST: Cannot put back an attribute that was removed.
  <xs:complexType name="opt_pro_ext">
    <xs:complexContent>
      <xs:extension base="opt_pro">
        <xs:attribute name="attr" type="xs:float"/>
      </xs:extension>
    </xs:complexContent>
  </xs:complexType>
-->
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0027 (unittest.TestCase):
    def setRattr_fixed (self, instance, value):
        instance.rattr_fixed = value

    def setAttr_fixed (self, instance, value):
        instance.attr_fixed = value

    def setAttr (self, instance, value):
        instance.attr = value

    def testRequired (self):
        self.assertEqual(2, len(req_struct._AttributeMap))
        i = ireq_struct()
        self.assertRaises(pyxb.MissingAttributeError, i.validateBinding)
        self.assertTrue(i.rattr is None)
        i.rattr = -4
        self.assertEqual(-4, i.rattr)
        self.assertTrue(i._AttributeMap['rattr'].provided(i))

        self.assertRaises(pyxb.MissingAttributeError, i.validateBinding) # Should fail because rattr_fixed was not explicitly set

        self.assertFalse(i._AttributeMap['rattr_fixed'].provided(i))
        self.assertEqual(30, i.rattr_fixed)

        self.assertRaises(pyxb.AttributeChangeError, self.setRattr_fixed, i, 41)
        self.assertFalse(i._AttributeMap['rattr_fixed'].provided(i))

        i.rattr_fixed = 30
        self.assertTrue(i._AttributeMap['rattr_fixed'].provided(i))
        self.assertEqual(30, i.rattr_fixed)
        self.assertTrue(i.validateBinding())

        self.assertRaises(pyxb.AttributeChangeError, self.setRattr_fixed, i, 41)

    def testRequiredCTor (self):
        i = ireq_struct(rattr=11, rattr_fixed=30)
        self.assertTrue(i.validateBinding())

        self.assertRaises(pyxb.AttributeChangeError, ireq_struct, rattr=11, rattr_fixed=31)

    def testOptional (self):
        self.assertEqual(3, len(opt_struct._AttributeMap))
        i = iopt_struct()

        self.assertTrue(i.attr is None)

        self.assertEqual(i._AttributeMap['attr'].dataType(), xs.int)

        self.assertFalse(i._AttributeMap['attr_def'].provided(i))
        self.assertEqual(10, i.attr_def)
        i.attr_def = 11
        self.assertEqual(11, i.attr_def)
        self.assertTrue(i._AttributeMap['attr_def'].provided(i))

        self.assertFalse(i._AttributeMap['attr_fixed'].provided(i))
        self.assertEqual(20, i.attr_fixed)

        self.assertRaises(pyxb.AttributeChangeError, self.setAttr_fixed, i, 21)
        self.assertFalse(i._AttributeMap['attr_fixed'].provided(i))
        self.assertEqual(20, i.attr_fixed)

        i.attr_fixed = 20
        self.assertTrue(i._AttributeMap['attr_fixed'].provided(i))
        self.assertEqual(20, i.attr_fixed)

        i.attr = 1000
        self.assertEqual(1000, i.attr)

    def testOptionalCtor (self):
        self.assertEqual(3, len(opt_struct._AttributeMap))
        self.assertRaises(pyxb.AttributeChangeError, opt_struct, attr_fixed=21)

        i = iopt_struct(attr=1, attr_def=2, attr_fixed=20)
        self.assertTrue(i.validateBinding())

        self.assertEqual(1, i.attr)
        self.assertEqual(2, i.attr_def)

    def testOptDef (self):
        self.assertEqual(3, len(opt_def._AttributeMap))
        self.assertNotEqual(opt_struct._AttributeMap['attr'], opt_def._AttributeMap['attr'])
        self.assertEqual(opt_struct._AttributeMap['attr'].key(), opt_def._AttributeMap['attr'].key())
        self.assertEqual(opt_struct._AttributeMap['attr_def'], opt_def._AttributeMap['attr_def'])
        self.assertEqual(opt_struct._AttributeMap['attr_fixed'], opt_def._AttributeMap['attr_fixed'])
        i = opt_def()
        self.assertEqual(5, i.attr)

    def testOptPro (self):
        self.assertEqual(3, len(opt_pro._AttributeMap))
        self.assertNotEqual(opt_struct._AttributeMap['attr'], opt_pro._AttributeMap['attr'])
        self.assertTrue(opt_pro._AttributeMap['attr'].prohibited())
        self.assertNotEqual(opt_struct._AttributeMap['attr_def'], opt_pro._AttributeMap['attr_def'])
        self.assertTrue(opt_pro._AttributeMap['attr_def'].prohibited())
        self.assertEqual(opt_struct._AttributeMap['attr_fixed'], opt_pro._AttributeMap['attr_fixed'])
        i = opt_pro()
        self.assertRaises(pyxb.ProhibitedAttributeError, lambda: i.attr)

    def testOptProCtor (self):
        self.assertRaises(pyxb.ProhibitedAttributeError, opt_pro, attr=1)

    def testOptRest (self):
        self.assertEqual(3, len(opt_rest._AttributeMap))
        i = opt_rest()

        self.assertEqual(i._AttributeMap['attr'].dataType(), xs.byte)
        self.assertNotEqual(opt_struct._AttributeMap['attr'], opt_rest._AttributeMap['attr'])
        self.assertEqual(opt_struct._AttributeMap['attr_def'], opt_rest._AttributeMap['attr_def'])
        self.assertEqual(opt_struct._AttributeMap['attr_fixed'], opt_rest._AttributeMap['attr_fixed'])

        self.assertRaises(pyxb.SimpleTypeValueError, self.setAttr, i, 1000)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="Number">
    <xs:simpleContent>
      <xs:extension base="xs:integer">
        <xs:attribute name="bounded" type="xs:boolean" default="true" />
      </xs:extension>
    </xs:simpleContent>
  </xs:complexType>
  <xs:element name="number" type="Number" nillable="true"/>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIssue0007 (unittest.TestCase):
    def testConstruction (self):
        # Absence of attribute uses implicit default
        i = number(53)
        self.assertEqual(six.u('<number>53</number>').encode('utf-8'), i.toxml('utf-8', root_only=True))
        self.assertTrue(i.bounded)
        # Explicit assignment of attribute makes it explicit even if same as default
        i.bounded = True
        self.assertEqual(six.u('<number bounded="true">53</number>').encode('utf-8'), i.toxml('utf-8', root_only=True))
        i.bounded = False
        self.assertEqual(six.u('<number bounded="false">53</number>').encode('utf-8'), i.toxml('utf-8', root_only=True))

    def testNilConstruction (self):
        i = number(_nil=True)
        self.assertEqual(six.u('<number xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>').encode('utf-8'), i.toxml('utf-8', root_only=True))
        self.assertRaises(pyxb.ContentInNilInstanceError, i._setValue, 25)

    def testReplacement (self):
        i = number(53, bounded=True)
        self.assertEqual(six.u('<number bounded="true">53</number>').encode('utf-8'), i.toxml('utf-8', root_only=True))
        self.assertEqual(53, i.value())
        i._setValue(27)
        self.assertEqual(six.u('<number bounded="true">27</number>').encode('utf-8'), i.toxml('utf-8', root_only=True))
        self.assertRaises(pyxb.SimpleTypeValueError, i._setValue, 'text')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
 <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">

  <xsd:complexType name="tAny">
    <xsd:all minOccurs="0">
      <xsd:element type="xsd:int" name="a" minOccurs="1"/>
      <xsd:element type="xsd:int" name="b" minOccurs="1"/>
    </xsd:all>
  </xsd:complexType>
  <xsd:element name="eAny" type="tAny"/>
</xsd:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0179 (unittest.TestCase):
    def testBasic (self):
        instance = CreateFromDocument("<eAny/>")
        self.assertTrue(instance.a is None)
        self.assertTrue(instance.b is None)
        instance = CreateFromDocument("<eAny><a>1</a><b>2</b></eAny>")
        self.assertEqual(instance.a, 1)
        self.assertEqual(instance.b, 2)
        instance = CreateFromDocument("<eAny><b>2</b><a>1</a></eAny>")
        self.assertEqual(instance.a, 1)
        self.assertEqual(instance.b, 2)
        self.assertRaises(pyxb.IncompleteElementContentError, CreateFromDocument, "<eAny><a>1</a></eAny>")

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="URN:test-trac-0047" xmlns:gml="URN:test-trac-0047">
        <simpleType name="NilReasonEnumeration">
                <union>
                        <simpleType>
                                <restriction base="string">
                                        <enumeration value="inapplicable"/>
                                        <enumeration value="missing"/>
                                        <enumeration value="template"/>
                                        <enumeration value="unknown"/>
                                        <enumeration value="withheld"/>
                                </restriction>
                        </simpleType>
                        <simpleType>
                                <restriction base="string">
                                        <pattern value="other:\w{2,}"/>
                                </restriction>
                        </simpleType>
                </union>
        </simpleType>
        <simpleType name="SignType">
                <annotation>
                        <documentation>gml:SignType is a convenience type with values "+" (plus) and "-" (minus).</documentation>
                </annotation>
                <restriction base="string">
                        <enumeration value="-"/>
                        <enumeration value="+"/>
                </restriction>
        </simpleType>
        <simpleType name="booleanOrNilReason">
                <annotation>
                        <documentation>Extension to the respective XML Schema built-in simple type to allow a choice of either a value of the built-in simple type or a reason for a nil value.</documentation>
                </annotation>
                <union memberTypes="gml:NilReasonEnumeration boolean anyURI"/>
        </simpleType>
        <simpleType name="doubleOrNilReason">
                <annotation>
                        <documentation>Extension to the respective XML Schema built-in simple type to allow a choice of either a value of the built-in simple type or a reason for a nil value.</documentation>
                </annotation>
                <union memberTypes="gml:NilReasonEnumeration double anyURI"/>
        </simpleType>
        <simpleType name="integerOrNilReason">
                <annotation>
                        <documentation>Extension to the respective XML Schema built-in simple type to allow a choice of either a value of the built-in simple type or a reason for a nil value.</documentation>
                </annotation>
                <union memberTypes="gml:NilReasonEnumeration integer anyURI"/>
        </simpleType>
  <element name="dblNil" type="gml:doubleOrNilReason"/>
</schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0047 (unittest.TestCase):
    def testEnum (self):
        self.assertEqual('missing', NilReasonEnumeration.Factory('missing'))
        self.assertRaises(pyxb.SimpleTypeValueError, NilReasonEnumeration.Factory, 'notValid')
        self.assertEqual('other:myReason', NilReasonEnumeration.Factory('other:myReason'))

    def testDblNil (self):
        v = dblNil(2.45)
        self.assertTrue(isinstance(v, float))
        v = dblNil('2.534')
        self.assertTrue(isinstance(v, float))
        v = dblNil('withheld')
        self.assertTrue(isinstance(v, NilReasonEnumeration._MemberTypes[0]))
        v = dblNil('other:myReason')
        self.assertTrue(isinstance(v, NilReasonEnumeration._MemberTypes[1]))
        v = dblNil('somethingElse')
        self.assertTrue(isinstance(v, xs.anyURI))

if __name__ == '__main__':
    unittest.main()



from pyxb.bundles.wssplat.raw.httpbind import *
import pyxb.bundles.wssplat.raw.httpbind as raw_httpbind
from pyxb.bundles.wssplat.wsdl11 import _WSDL_binding_mixin, _WSDL_port_mixin, _WSDL_operation_mixin

class bindingType (raw_httpbind.bindingType, _WSDL_binding_mixin):
    pass
raw_httpbind.bindingType._SetSupersedingClass(bindingType)

class addressType (raw_httpbind.addressType, _WSDL_port_mixin):
    pass
raw_httpbind.addressType._SetSupersedingClass(addressType)

class operationType (raw_httpbind.operationType, _WSDL_operation_mixin):
    def locationInformation (self):
        return self.location()
raw_httpbind.operationType._SetSupersedingClass(operationType)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:complexType name="tBase">
  <xs:sequence>
    <xs:any minOccurs="0" maxOccurs="unbounded"/>
  </xs:sequence>
  <xs:attribute name="attr" type="xs:NMTOKENS" use="optional"/>
</xs:complexType>
<xs:complexType name="tExt">
  <xs:complexContent>
   <xs:restriction base="tBase">
     <xs:attribute name="eattr" type="xs:string" use="required"/>
     <xs:attribute name="attr" type="xs:NMTOKENS" use="prohibited"/>
   </xs:restriction>
  </xs:complexContent>
</xs:complexType>
<xs:element name="base" type="tBase"/>
<xs:element name="ext" type="tExt"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest
import xml.dom

class TestTrac_0053 (unittest.TestCase):
    def testBase (self):
        xmls = '<base attr="value"/>'
        instance = CreateFromDocument(xmls)
        self.assertTrue(isinstance(instance.attr, list))
        self.assertEqual(1, len(instance.attr))
        self.assertEqual("value", instance.attr[0])

    def testExt (self):
        xmls = '<ext eattr="value"/>'
        instance = CreateFromDocument(xmls)
        self.assertEqual("value", instance.eattr)
        # Creation from DOM takes a different code path
        domn = pyxb.utils.domutils.StringToDOM(xmls)
        instance = CreateFromDOM(domn)
        self.assertEqual("value", instance.eattr)

    def testExtMissingRequired (self):
        xmls = '<ext/>'
        self.assertRaises(pyxb.MissingAttributeError, CreateFromDocument, xmls)
        # Creation from DOM takes a different code path
        domn = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.MissingAttributeError, CreateFromDOM, domn)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import mr
import unittest
import pyxb
import pyxb.binding.datatypes as xsd

class TestTrac0080 (unittest.TestCase):

    _NotANormalizedString = "\nmulti\nline\ttabbed\n"
    _NotAToken = ' leading spaces '
    _NotAnNCName = 'internal spaces'
    _NCName = 'simple'

    def assignAttribute_ (self, instance, value):
        instance.anAttribute = value

    def testType4 (self): # base
        i4 = mr.Type4()
        au = i4._AttributeMap.get('anAttribute')
        self.assertEqual(au.dataType(), xsd.normalizedString)
        self.assertFalse(au.required())
        self.assertRaises(pyxb.SimpleTypeValueError, self.assignAttribute_, i4, self._NotANormalizedString)

    def testType3 (self): # restrict type
        i3 = mr.Type3()
        au = i3._AttributeMap.get('anAttribute')
        self.assertEqual(au.dataType(), xsd.token)
        self.assertNotEqual(au, mr.Type4._AttributeMap.get(au.name()))
        self.assertFalse(au.required())
        #self.assertRaises(pyxb.SimpleTypeValueError, self.assignAttribute_, i3, self._NotAToken)
        self.assignAttribute_(i3, self._NotAnNCName)
        self.assertEqual(self._NotAnNCName, i3.anAttribute)

    def testType2 (self): # extend isSet
        i2 = mr.Type2()
        au = i2._AttributeMap.get('anAttribute')
        self.assertEqual(au.dataType(), xsd.token)
        self.assertEqual(au, mr.Type3._AttributeMap.get(au.name()))
        self.assertFalse(au.required())

    def testType1 (self): # restrict type
        i1 = mr.Type1()
        au = i1._AttributeMap.get('anAttribute')
        self.assertEqual(au.dataType(), xsd.NCName)
        self.assertFalse(au.required())
        # The whiteSpace facet on xsd:token is collapse, which does
        # not remove the interior space.
        self.assertRaises(pyxb.SimpleTypeValueError, self.assignAttribute_, i1, self._NotAToken)
        self.assertRaises(pyxb.SimpleTypeValueError, self.assignAttribute_, i1, self._NotAnNCName)
        self.assignAttribute_(i1, self._NCName)
        self.assertEqual(self._NCName, i1.anAttribute)

    def testRoot (self): # restrict required
        r = mr.root()
        rt = type(r)
        au = rt._AttributeMap.get('anAttribute')
        self.assertEqual(au.dataType(), xsd.NCName)
        self.assertTrue(au.required())
        self.assertRaises(pyxb.MissingAttributeError, r.validateBinding)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import pyxb
import sample
from pyxb.namespace.builtin import XMLSchema_instance as xsi

class TestTrac0202 (unittest.TestCase):
    def tearDown (self):
        pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(sample.Namespace)

    Expectedt = """<?xml version="1.0" encoding="utf-8"?>
<samplerootelement xmlns="http://sample" xmlns:ns1="http://sample" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="c:\sample.xsd">
\t<sampleelement>
\t\t<ValueAmount ns1:currencyID="abc">100.0</ValueAmount>
\t</sampleelement>
</samplerootelement>
"""
    Expectedd = Expectedt.encode('utf-8')

    def testIssue (self):
        elm = sample.sampleelementType()
        elm.ValueAmount = '100'
        elm.ValueAmount.currencyID = 'abc'
        sam = sample.samplerootelement()
        sam.sampleelement.append(elm)
        bds = pyxb.utils.domutils.BindingDOMSupport()
        bds.setDefaultNamespace(sample.Namespace)
        bds.declareNamespace(xsi)
        samdom = sam.toDOM(bds)
        bds.addAttribute(samdom.documentElement, xsi.createExpandedName('schemaLocation'), "c:\sample.xsd")
        # xsi is probably not referenced elsewhere, so add the XMLNS declaration too
        bds.addXMLNSDeclaration(samdom.documentElement, xsi)
        xmld = samdom.toprettyxml(encoding = "utf-8")
        self.assertEqual(self.Expectedd, xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.datatypes as xs

import unittest

class TestTrac_0138 (unittest.TestCase):
    Literal = '2012-06-07T07:14:38.125296'
    def testPreservation (self):
        value = xs.dateTime(self.Literal)
        self.assertEqual(value.xsdLiteral(), self.Literal)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-mg-sequence.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils
def ToDOM (instance, tag=None):
    return instance.toDOM().documentElement

import unittest
import collections

class TestMGSeq (unittest.TestCase):
    def setUp (self):
        # Hide the warning about failure to convert DOM node {}third
        # to a binding
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel

    def testBad (self):
        # Second is wrong element tag
        # Hide warning about failure to convert
        self.__basis_log.level = logging.ERROR
        xml = '<ns1:wrapper xmlns:ns1="URN:test-mg-sequence"><first/><second/><third/><fourth_0_2/></ns1:wrapper>'
        dom = pyxb.utils.domutils.StringToDOM(xml)
        self.assertRaises(UnrecognizedContentError, wrapper.createFromDOM, dom.documentElement)

    def testBasics (self):
        xmlt = six.u('<ns1:wrapper xmlns:ns1="URN:test-mg-sequence"><first/><second_opt/><third/><fourth_0_2/></ns1:wrapper>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = wrapper.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, sequence._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.second_opt, sequence._ElementMap['second_opt'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.third, sequence._ElementMap['third'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.fourth_0_2, collections.MutableSequence))
        self.assertEqual(1, len(instance.fourth_0_2))
        self.assertTrue(isinstance(instance.fourth_0_2[0], sequence._ElementMap['fourth_0_2'].elementBinding().typeDefinition()))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMultiplesAtEnd (self):
        xmlt = six.u('<ns1:wrapper xmlns:ns1="URN:test-mg-sequence"><first/><third/><fourth_0_2/><fourth_0_2/></ns1:wrapper>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = wrapper.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, sequence._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(instance.second_opt is None)
        self.assertTrue(isinstance(instance.third, sequence._ElementMap['third'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.fourth_0_2, collections.MutableSequence))
        self.assertEqual(2, len(instance.fourth_0_2))
        self.assertTrue(isinstance(instance.fourth_0_2[0], sequence._ElementMap['fourth_0_2'].elementBinding().typeDefinition()))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMultiplesInMiddle (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><first/><second_multi/><second_multi/><third/></ns1:altwrapper>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = altwrapper.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, collections.MutableSequence))
        self.assertEqual(1, len(instance.first))
        self.assertEqual(2, len(instance.second_multi))
        self.assertTrue(isinstance(instance.third, altsequence._ElementMap['third'].elementBinding().typeDefinition()))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMultiplesAtStart (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><first/><first/><third/></ns1:altwrapper>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = altwrapper.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, collections.MutableSequence))
        self.assertEqual(2, len(instance.first))
        self.assertEqual(0, len(instance.second_multi))
        self.assertTrue(isinstance(instance.third, altsequence._ElementMap['third'].elementBinding().typeDefinition()))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)
        instance = altwrapper(first=[ altsequence._ElementMap['first'].elementBinding()(), altsequence._ElementMap['first'].elementBinding()() ], third=altsequence._ElementMap['third'].elementBinding()())
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMissingInMiddle (self):
        xmlt = six.u('<ns1:wrapper xmlns:ns1="URN:test-mg-sequence"><first/><third/></ns1:wrapper>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = wrapper.createFromDOM(dom.documentElement)
        self.assertTrue(isinstance(instance.first, sequence._ElementMap['first'].elementBinding().typeDefinition()))
        self.assertTrue(instance.second_opt is None)
        self.assertTrue(isinstance(instance.third, sequence._ElementMap['third'].elementBinding().typeDefinition()))
        self.assertTrue(isinstance(instance.fourth_0_2, collections.MutableSequence))
        self.assertEqual(0, len(instance.fourth_0_2))
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testMissingAtStart (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><third/></ns1:altwrapper>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, altwrapper.createFromDOM, dom.documentElement)
        instance = altwrapper(third=altsequence._ElementMap['third'].elementBinding()())
        self.assertRaises(pyxb.IncompleteElementContentError, ToDOM, instance)

    def testMissingAtEndLeadingContent (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><first/></ns1:altwrapper>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, altwrapper.createFromDOM, dom.documentElement)

    def testMissingAtEndNoContent (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"></ns1:altwrapper>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, altwrapper.createFromDOM, dom.documentElement)

    def testTooManyAtEnd (self):
        xmlt = six.u('<ns1:wrapper xmlns:ns1="URN:test-mg-sequence"><first/><third/><fourth_0_2/><fourth_0_2/><fourth_0_2/></ns1:wrapper>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, wrapper.createFromDOM, dom.documentElement)

    def testTooManyAtStart (self):
        xmlt = six.u('<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><first/><first/><first/><third/></ns1:altwrapper>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, altwrapper.createFromDOM, dom.documentElement)
        instance = altwrapper(first=[ altsequence._ElementMap['first'].elementBinding()(), altsequence._ElementMap['first'].elementBinding()(), altsequence._ElementMap['first'].elementBinding()() ], third=altsequence._ElementMap['third'].elementBinding()())
        self.assertRaises(pyxb.UnprocessedElementContentError, ToDOM, instance)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(UnprocessedElementContentError) as cm:
                instance.toxml('utf-8')
            # Verify the exception tells us what was being processed
            self.assertEqual(instance, cm.exception.instance)
            # Verify the exception tells us what was left over
            first_ed = altsequence._ElementMap['first']
            self.assertEqual(instance.first[2], cm.exception.symbol_set[first_ed][0])

    def testTooManyInMiddle (self):
        xml = '<ns1:altwrapper xmlns:ns1="URN:test-mg-sequence"><second_multi/><second_multi/><second_multi/><third/></ns1:altwrapper>'
        dom = pyxb.utils.domutils.StringToDOM(xml)
        self.assertRaises(UnrecognizedContentError, altwrapper.createFromDOM, dom.documentElement)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import time
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils.six.moves import xrange

max_reps = 20

def buildTest (num_reps, constraint='minOccurs="0" maxOccurs="1"'):
    edefs = []
    cdefs = []
    duse = []
    for r in xrange(num_reps):
        edefs.append('<xs:element name="rep%d" type="xs:string"/>' % (r,))
        cdefs.append('<xs:element ref="rep%d" %s/>' % (r, constraint))
        duse.append('<rep%d>text_%d</rep%d>' % (r, r, r))

    schema = ''.join([ '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">''',
                   "\n".join(edefs),
                   '''<xs:element name="collection">
<xs:complexType><xs:sequence>''',
                   "\n".join(cdefs),
                   '''</xs:sequence></xs:complexType>
</xs:element>
</xs:schema>''' ])

    xmls = '<collection>' + ''.join(duse) + '</collection>'

    return (schema, xmls)

for size in xrange(1, max_reps):
    (schema, xmls) = buildTest(size)

    t0 = time.time()
    code = pyxb.binding.generate.GeneratePython(schema_text=schema)
    t1 = time.time()
    rv = compile(code, 'test', 'exec')
    t2 = time.time()
    eval(rv)
    t3 = time.time()
    #open('code.py', 'w').write(code)
    #print xmls
    ct0 = time.time()
    doc = CreateFromDocument(xmls)
    ct1 = time.time()

    print("%d gen=%g cpl=%g ld=%g prs=%g" % (size, t1 - t0, t2 - t1, t3 - t2, ct1 - ct0))
    # Should not take more than a second (really, less than 10ms)
    assert (ct1 - ct0) < 1.0
    #open('code.py', 'w').write(code)



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Functions that aid with generating text from templates and maps."""

import re

# POSIX shell variable syntax:
#  Expansions with unset var
#  ${var}=
#  ${var+WORD}=
#  ${var:+WORD}=
#  ${var-WORD}=WORD
#  ${var:-WORD}=WORD
#  Expansions with empty var
#  ${var}=
#  ${var+WORD}=WORD
#  ${var:+WORD}=
#  ${var-WORD}=
#  ${var:-WORD}=WORD
#  Expansions with var=SET
#  ${var}=SET
#  ${var+WORD}=WORD
#  ${var:+WORD}=WORD
#  ${var-WORD}=SET
#  ${var:-WORD}=SET

# This expression replaces markers in template text with the value
# obtained by looking up the marker in a dictionary.
# %{id} = value
_substIdPattern = re.compile("%{(?P<id>\w+)}")

# This expression performs conditional substitution: if the expression
# provided evaluates to true in a given context, then one value is
# substituted, otherwise the alternative value is substituted.
# %{?<cond>??<true>?:<false>?}
# %{?1 == 2??true?:false?}
_substConditionalPattern = re.compile("%{\?(?P<expr>.+?)\?\?(?P<true>.*?)(\?:(?P<false>.*?))?\?}", re.MULTILINE + re.DOTALL)

# This expression tests whether an identifier is defined to a non-None
# value in the context; if so, it replaces the marker with template
# text.  In that replacement text, the value ?@ is replaced by the
# test expression.  Contrast POSIX shell ${ID+subst}${ID-subst}
# Note: NOT by the value of the test expression.  If no replacement
# text is given, the replacement '%{?@}' is used, which replaces it
# with the value of the test expression.
# %{?<id>?+<yessubst>?-?<nosubst>}}
# %{?maybe_text?+?@ is defined to be %{?@}?}
_substIfDefinedPattern = re.compile("%{\?(?P<id>\w+)(\?\+(?P<repl>.*?))?(\?\-(?P<ndrepl>.*?))?\?}", re.MULTILINE + re.DOTALL)

# The pattern which, if present in the body of a IfDefined block, is
# replaced by the test expression.
_substDefinedBodyPattern = re.compile("\?@")

def _bodyIfDefinedPattern (match_object, dictionary):
    global _substDefinedBodyPattern
    id = match_object.group('id')
    repl = match_object.group('repl')
    ndrepl = match_object.group('ndrepl')
    value = dictionary.get(id)
    if value is not None:
        if repl:
            return _substDefinedBodyPattern.sub(id, repl)
        if ndrepl:
            return ''
        return _substDefinedBodyPattern.sub(id, '%{?@}')
    else:
        if ndrepl:
            return _substDefinedBodyPattern.sub(id, ndrepl)
        return ''

def _bodyConditionalPattern (match_object, dictionary):
    global _substDefinedBodyPattern
    expr = match_object.group('expr')
    true = match_object.group('true')
    false = match_object.group('false')
    value = None
    try:
        value = eval(expr, dictionary)
    except Exception as e:
        return '%%{EXCEPTION: %s}' % (e,)
    if value:
        return _substDefinedBodyPattern.sub(expr, true)
    if false is not None:
        return _substDefinedBodyPattern.sub(expr, false)
    return ''

def replaceInText (text, **dictionary):
    global _substIfDefinedPattern
    global _substConditionalPattern
    global _substIdPattern
    global _substDefinedBodyPattern
    rv = text
    rv = _substIfDefinedPattern.sub(lambda _x: _bodyIfDefinedPattern(_x, dictionary), rv)
    rv = _substConditionalPattern.sub(lambda _x: _bodyConditionalPattern(_x, dictionary), rv)
    rv =  _substIdPattern.sub(
        lambda _x,_map=dictionary:
           _map.get(_x.group('id'), '%%{MISSING:%s}' % (_x.group('id'),))
        , rv)
    return rv



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:element name="testElt">
  <xs:complexType>
  <xs:sequence>
    <xs:element name="content" type="xs:string"/>
  </xs:sequence>
  </xs:complexType>
</xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0051 (unittest.TestCase):
    def __setContent (self, e, v):
        e.content = v

    def __setToXML (self, e, v):
        e.toxml = v

    def testElement (self):
        e = testElt('hello')
        self.assertEqual('hello', e.content_)
        e.content_ = 'goodbye'
        self.assertEqual('goodbye', e.content_)
        if pyxb._CorruptionDetectionEnabled:
            self.assertRaises(pyxb.BindingError, self.__setContent, e, 'invalid')
            self.assertRaises(pyxb.BindingError, self.__setToXML, e, 32)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import app
import common

import pyxb.utils.domutils
from pyxb.utils import six

pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(app.Namespace, 'app')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(common.Namespace, 'common')

class Test (unittest.TestCase):

    def testMissingApp (self):
        # The app element is not in the base common, it's in the
        # application-specific module that's private.
        self.assertRaises(AttributeError, getattr, common, 'app')

    def testApp (self):
        instance = app.elt(base='hi', app='there')
        xmlt = six.u('<app:elt xmlns:app="urn:app" xmlns:common="urn:common"><common:base><common:bstr>hi</common:bstr></common:base><common:app><common:astr>there</common:astr></common:app></app:elt>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="structure">
    <xs:complexType><xs:anyAttribute processContents="lax"/></xs:complexType>
  </xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

AttributeNamespace = pyxb.namespace.NamespaceInstance('URN:attr:200908231005')

class TestTrac_200908231005 (unittest.TestCase):
    def testParsing (self):
        xmls = '<structure xmlns:attr="%s" attr:field="value"/>' % (AttributeNamespace.uri(),)
        instance = CreateFromDocument(xmls)
        wam = instance.wildcardAttributeMap()
        self.assertEqual(1, len(wam))
        self.assertEqual('value', wam.get(AttributeNamespace.createExpandedName('field')))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

# Derived from test-trac-0182 which uses wildcards
import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element type="xs:int" name="eOne"/>
  <xs:element type="xs:int" name="eTwo"/>
  <xs:complexType name="tElt">
    <xs:sequence>
      <xs:element ref="eOne" minOccurs="0"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="elt" type="tElt"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0189 (unittest.TestCase):
    def tearDown (self):
        pyxb.RequireValidWhenGenerating(True)
        pyxb.RequireValidWhenParsing(True)

    def testGenerate (self):
        ia = eOne(1)
        ib = eTwo(2)
        content = [ ia, ib ]
        i = elt(ia)
        self.assertRaises(UnrecognizedContentError, elt, ia, ib)

    def testParse (self):
        xmls = '<elt><eOne>1</eOne></elt>'
        i = CreateFromDocument(xmls)
        self.assertEqual(i.eOne, 1)
        xmls = '<elt><eOne>1</eOne><eTwo>2</eTwo></elt>'
        self.assertRaises(UnrecognizedContentError, CreateFromDocument, xmls)
        pyxb.RequireValidWhenParsing(False)
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.StructuralBadDocumentError, CreateFromDocument, xmls)
        else:
            with self.assertRaises(pyxb.StructuralBadDocumentError) as cm:
                i = CreateFromDocument(xmls)
            e = cm.exception

    def testAppend (self):
        i = elt(1)
        self.assertEqual(i.eOne, 1)
        ed = i._ElementMap.get('eOne')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.NonPluralAppendError, ed.append, i, 2)
        else:
            with self.assertRaises(pyxb.NonPluralAppendError) as cm:
                ed.append(i, 2)
            e = cm.exception
            self.assertEqual(e.instance, i)
            self.assertEqual(e.element_declaration, ed)
            self.assertEqual(e.value, 2)
            self.assertEqual(e.details(), 'Instance of tElt cannot append to element eOne')

if __name__ == '__main__':
    unittest.main()
